From 3644c481e712db246a157db43529ad2a7c0f8d75 Mon Sep 17 00:00:00 2001
From: Roberto Corno <corno.roberto@gmail.com>
Date: Thu, 30 Aug 2012 12:54:27 +0200
Subject: [PATCH 4/5] Add 0x4F Descriptor Decoder/Generator support

---
 src/descriptors/dr_4f.c |  101 +++++++++++++++++++++++++++++++++++++++++++++++
 src/descriptors/dr_4f.h |   99 ++++++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 200 insertions(+), 0 deletions(-)
 create mode 100644 src/descriptors/dr_4f.c
 create mode 100644 src/descriptors/dr_4f.h

diff --git a/src/descriptors/dr_4f.c b/src/descriptors/dr_4f.c
new file mode 100644
index 0000000..ba70790
--- /dev/null
+++ b/src/descriptors/dr_4f.c
@@ -0,0 +1,101 @@
+/*
+ * dr_4f.c
+ * Copyright (C) 2012 VideoLAN
+ *
+ * Authors: Roberto Corno <corno.roberto@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ *****************************************************************************/
+
+#include "config.h"
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <stdbool.h>
+#include <string.h>
+
+#if defined(HAVE_INTTYPES_H)
+#include <inttypes.h>
+#elif defined(HAVE_STDINT_H)
+#include <stdint.h>
+#endif
+
+#include "../dvbpsi.h"
+#include "../dvbpsi_private.h"
+#include "../descriptor.h"
+
+#include "dr_4f.h"
+
+/*****************************************************************************
+ * dvbpsi_DecodeTimeShiftedEventDr
+ *****************************************************************************/
+dvbpsi_tshifted_ev_dr_t* dvbpsi_DecodeTimeShiftedEventDr(dvbpsi_descriptor_t * p_descriptor) {
+    /* Check the tag */
+    if (p_descriptor->i_tag != 0x4F)
+        return NULL;
+
+    /* Don't decode twice */
+    if (p_descriptor->p_decoded)
+        return p_descriptor->p_decoded;
+
+    /* Check the length */
+    if (p_descriptor->i_length < 4)
+        return NULL;
+
+    /* Allocate memory */
+    dvbpsi_tshifted_ev_dr_t * p_decoded;
+    p_decoded = (dvbpsi_tshifted_ev_dr_t*)calloc(1, sizeof(dvbpsi_tshifted_ev_dr_t));
+    if (!p_decoded)
+        return NULL;
+
+    /* Decode data */
+    p_decoded->i_ref_service_id = p_descriptor->p_data[0] << 8
+                                | p_descriptor->p_data[1];
+    p_decoded->i_ref_event_id = p_descriptor->p_data[2] << 8
+                                | p_descriptor->p_data[3];
+
+    p_descriptor->p_decoded = (void*)p_decoded;
+
+    return p_decoded;
+}
+
+/*****************************************************************************
+ * dvbpsi_GenTimeShiftedEventDr
+ *****************************************************************************/
+dvbpsi_descriptor_t *dvbpsi_GenTimeShiftedEventDr(dvbpsi_tshifted_ev_dr_t * p_decoded,
+                                                  bool b_duplicate) {
+    /* Create the descriptor */
+    dvbpsi_descriptor_t * p_descriptor =
+            dvbpsi_NewDescriptor(0x4f, 4, NULL);
+    if (!p_descriptor)
+        return NULL;
+
+    /* Encode data */
+	p_descriptor->p_data[0] = p_decoded->i_ref_service_id >> 8;
+    p_descriptor->p_data[1] = p_decoded->i_ref_service_id;
+	p_descriptor->p_data[2] = p_decoded->i_ref_event_id >> 8;
+    p_descriptor->p_data[3] = p_decoded->i_ref_event_id;
+
+    if (b_duplicate)
+    {
+        /* Duplicate decoded data */
+        p_descriptor->p_decoded =
+               dvbpsi_DuplicateDecodedDescriptor(p_decoded,
+                      sizeof(dvbpsi_tshifted_ev_dr_t));
+    }
+
+    return p_descriptor;
+}
diff --git a/src/descriptors/dr_4f.h b/src/descriptors/dr_4f.h
new file mode 100644
index 0000000..9ef261f
--- /dev/null
+++ b/src/descriptors/dr_4f.h
@@ -0,0 +1,99 @@
+/*****************************************************************************
+ * dr_4f.h
+ * Copyright (C) 2012 VideoLAN
+ *
+ * Authors: Roberto Corno <corno.roberto@gmail.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ *****************************************************************************/
+
+/*!
+ * \file <dr_4f.h>
+ * \author Corno Roberto <corno.roberto@gmail.com>
+ * \brief Application interface for the time shifted event
+ * descriptor decoder and generator.
+ *
+ * Application interface for the DVB time shifted event descriptor
+ * descriptor decoder and generator. This descriptor's definition can be found in
+ * ETSI EN 300 468 section 6.2.44.
+ */
+
+#ifndef DR_4F_H_
+#define DR_4F_H_
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/*****************************************************************************
+ * dvbpsi_tshifted_ev_dr_t
+ *****************************************************************************/
+/*!
+ * \struct dvbpsi_tshifted_ev_dr_t
+ * \brief "time shifted event" descriptor structure.
+ *
+ * This structure is used to store a decoded "time shifted event"
+ * descriptor. (ETSI EN 300 468 section 6.2.44).
+ */
+/*!
+ * \typedef struct dvbpsi_tshifted_ev_dr_s dvbpsi_tshifted_ev_dr_t
+ * \brief dvbpsi_tshifted_ev_dr_t type definition.
+ */
+/*!
+ * \struct dvbpsi_tshifted_ev_dr_s
+ * \brief struct dvbpsi_tshifted_ev_dr_s @see dvbpsi_tshifted_ev_dr_t
+ */
+typedef struct dvbpsi_tshifted_ev_dr_s
+{
+  uint16_t       i_ref_service_id;         /*!< reference service id */
+  uint16_t       i_ref_event_id;           /*!< reference service id */
+} dvbpsi_tshifted_ev_dr_t;
+
+/*****************************************************************************
+ * dvbpsi_DecodeTimeShiftedEventDr
+ *****************************************************************************/
+/*!
+ * \fn dvbpsi_tshifted_ev_dr_t * dvbpsi_DecodeTimeShiftedEventDr(
+                                        dvbpsi_descriptor_t * p_descriptor)
+ * \brief "time shifted event" descriptor decoder.
+ * \param p_descriptor pointer to the descriptor structure
+ * \return a pointer to a new "time shifted event" descriptor structure
+ * which contains the decoded data.
+ */
+dvbpsi_tshifted_ev_dr_t* dvbpsi_DecodeTimeShiftedEventDr(dvbpsi_descriptor_t * p_descriptor);
+
+/*****************************************************************************
+ * dvbpsi_GenTimeShiftedEventDr
+ *****************************************************************************/
+/*!
+ * \fn dvbpsi_descriptor_t *dvbpsi_GenTimeShiftedEventDr(dvbpsi_tshifted_ev_dr_t *p_decoded,
+                                                         bool b_duplicate);
+ * \brief "time shifted event" descriptor generator.
+ * \param p_decoded pointer to a decoded "time shifted event" descriptor structure
+ * \param b_duplicate if true then duplicate the p_decoded structure into
+ * the descriptor
+ * \return a pointer to a new descriptor structure which contains encoded data.
+ */
+dvbpsi_descriptor_t *dvbpsi_GenTimeShiftedEventDr(dvbpsi_tshifted_ev_dr_t * p_decoded,
+                                                  bool b_duplicate);
+
+#ifdef __cplusplus
+};
+#endif
+
+#else
+#error "Multiple inclusions of dr_4f.h"
+#endif /* DR_4F_H_ */
-- 
1.7.5.4
